@@ -41,7 +41,6 @@ module Network.TLS.Extension
     , availableEllipticCurves
     ) where
 
-import Control.Applicative ((<$>),(<*>))
 import Control.Monad
 
 import Data.Word
@@ -53,6 +52,7 @@ import qualified Data.ByteString.Char8 as BC
 import Network.TLS.Extension.EC
 import Network.TLS.Struct (ExtensionID, EnumSafe8(..), EnumSafe16(..), HashAndSignatureAlgorithm)
 import Network.TLS.Wire
+import Network.TLS.Imports
 import Network.TLS.Packet (putSignatureHashAlgorithm, getSignatureHashAlgorithm)
 
 type HostName = String
@@ -258,21 +258,32 @@ data EllipticCurvesSupported = EllipticCurvesSupported [NamedCurve]
 
 data NamedCurve =
       SEC CurveName
+    | BrainPool BrainPoolCurve
     | NamedCurve_arbitrary_explicit_prime_curves
     | NamedCurve_arbitrary_explicit_char2_curves
     deriving (Show,Eq)
 
--- FIXME: currently maximum crypto strength of our supported
---        cipher suite is 128 bits. Not support 384 and 512.
+data BrainPoolCurve =
+      BrainPoolP512R1 -- 28
+    | BrainPoolP384R1 -- 27
+    | BrainPoolP256R1 -- 26
+    deriving (Show,Eq)
+
 availableEllipticCurves :: [NamedCurve]
-availableEllipticCurves = [SEC SEC_p160r1, SEC SEC_p224r1, SEC SEC_p256r1]
+availableEllipticCurves = [SEC SEC_p256r1, SEC SEC_p521r1]
 
 instance EnumSafe16 NamedCurve where
     fromEnumSafe16 NamedCurve_arbitrary_explicit_prime_curves = 0xFF01
     fromEnumSafe16 NamedCurve_arbitrary_explicit_char2_curves = 0xFF02
     fromEnumSafe16 (SEC nc) = maybe (error "named curve: internal error") id $ fromCurveName nc
+    fromEnumSafe16 (BrainPool BrainPoolP512R1) = 28
+    fromEnumSafe16 (BrainPool BrainPoolP384R1) = 27
+    fromEnumSafe16 (BrainPool BrainPoolP256R1) = 26
     toEnumSafe16 0xFF01 = Just NamedCurve_arbitrary_explicit_prime_curves
     toEnumSafe16 0xFF02 = Just NamedCurve_arbitrary_explicit_char2_curves
+    toEnumSafe16 26     = Just (BrainPool BrainPoolP256R1)
+    toEnumSafe16 27     = Just (BrainPool BrainPoolP384R1)
+    toEnumSafe16 28     = Just (BrainPool BrainPoolP512R1)
     toEnumSafe16 n      = SEC <$> toCurveName n
 
 -- on decode, filter all unknown curves
