@@ -19,6 +19,7 @@ module Network.TLS.Handshake.Signature
 import Network.TLS.Crypto
 import Network.TLS.Context.Internal
 import Network.TLS.Struct
+import Network.TLS.Imports
 import Network.TLS.Packet (generateCertificateVerify_SSL, encodeSignedDHParams, encodeSignedECDHParams)
 import Network.TLS.Parameters (supportedHashSignatures)
 import Network.TLS.State
@@ -26,7 +27,6 @@ import Network.TLS.Handshake.State
 import Network.TLS.Handshake.Key
 import Network.TLS.Util
 
-import Control.Applicative
 import Control.Monad.State
 
 certificateVerifyCheck :: Context
@@ -79,10 +79,11 @@ signatureHashData :: SignatureAlgorithm -> Maybe HashAlgorithm -> Hash
 signatureHashData SignatureRSA mhash =
     case mhash of
         Just HashSHA512 -> SHA512
+        Just HashSHA384 -> SHA384
         Just HashSHA256 -> SHA256
         Just HashSHA1   -> SHA1
         Nothing         -> SHA1_MD5
-        Just hash       -> error ("unimplemented RSA signature hash type: " ++ show hash)
+        Just hsh        -> error ("unimplemented RSA signature hash type: " ++ show hsh)
 signatureHashData SignatureDSS mhash =
     case mhash of
         Nothing       -> SHA1
@@ -95,7 +96,7 @@ signatureHashData SignatureECDSA mhash =
         Just HashSHA256 -> SHA256
         Just HashSHA1   -> SHA1
         Nothing         -> SHA1_MD5
-        Just hash       -> error ("unimplemented ECDSA signature hash type: " ++ show hash)
+        Just hsh        -> error ("unimplemented ECDSA signature hash type: " ++ show hsh)
 signatureHashData sig _ = error ("unimplemented signature type: " ++ show sig)
 
 --signatureCreate :: Context -> Maybe HashAndSignatureAlgorithm -> HashDescr -> Bytes -> IO DigitallySigned
