@@ -30,6 +30,7 @@ import Network.TLS.Wire
 import Network.TLS.Packet
 import Data.ByteString (ByteString)
 import qualified Data.ByteString as B
+import qualified Data.ByteArray as B (convert)
 
 disengageRecord :: Record Ciphertext -> RecordM (Record Plaintext)
 disengageRecord = decryptRecord >=> uncompressRecord
@@ -84,7 +85,10 @@ decryptData ver record econtent tst = decryptOf (cstKey cst)
         decryptOf :: BulkState -> RecordM Bytes
         decryptOf (BulkStateBlock decryptF) = do
             let minContent = (if explicitIV then bulkIVSize bulk else 0) + max (macSize + 1) blockSize
+
+            -- check if we have enough bytes to cover the minimum for this cipher
             when ((econtentLen `mod` blockSize) /= 0 || econtentLen < minContent) $ sanityCheckError
+
             {- update IV -}
             (iv, econtent') <- if explicitIV
                                   then get2 econtent (bulkIVSize bulk, econtentLen - bulkIVSize bulk)
@@ -102,7 +106,9 @@ decryptData ver record econtent tst = decryptOf (cstKey cst)
                     }
 
         decryptOf (BulkStateStream (BulkStream decryptF)) = do
+            -- check if we have enough bytes to cover the minimum for this cipher
             when (econtentLen < macSize) $ sanityCheckError
+
             let (content', bulkStream') = decryptF econtent
             {- update Ctx -}
             let contentlen        = B.length content' - macSize
@@ -115,18 +121,22 @@ decryptData ver record econtent tst = decryptOf (cstKey cst)
                     }
 
         decryptOf (BulkStateAEAD decryptF) = do
-            let authtaglen = 16 -- FIXME: fixed_iv_length + record_iv_length
-                nonceexplen = 8 -- FIXME: record_iv_length
-                econtentlen = B.length econtent - authtaglen - nonceexplen
-            (enonce, econtent', authTag) <- get3 econtent (nonceexplen, econtentlen, authtaglen)
+            let authTagLen  = bulkAuthTagLen bulk
+                nonceExpLen = bulkExplicitIV bulk
+                cipherLen   = econtentLen - authTagLen - nonceExpLen
+
+            -- check if we have enough bytes to cover the minimum for this cipher
+            when (econtentLen < (authTagLen + nonceExpLen)) $ sanityCheckError
+
+            (enonce, econtent', authTag) <- get3 econtent (nonceExpLen, cipherLen, authTagLen)
             let encodedSeq = encodeWord64 $ msSequence $ stMacState tst
                 Header typ v _ = recordToHeader record
-                hdr = Header typ v $ fromIntegral econtentlen
+                hdr = Header typ v $ fromIntegral cipherLen
                 ad = B.concat [ encodedSeq, encodeHeader hdr ]
                 nonce = cstIV (stCryptState tst) `B.append` enonce
                 (content, authTag2) = decryptF nonce econtent' ad
 
-            when (AuthTag authTag /= authTag2) $
+            when (AuthTag (B.convert authTag) /= authTag2) $
                 throwError $ Error_Protocol ("bad record mac", True, BadRecordMac)
 
             modify incrRecordState
